// dynamiczny stos liczb typu int, (c) Z. Koza, 2005

#include <cstddef>       // <-- definicja size_t
#include <cassert>       // <-- definicja assert
#include <iostream>

class Stos
{
public:
    Stos ();              // konstruktor domylny
    Stos (Stos const&);   // konstruktor kopiujcy
    ~Stos();              // destruktor
    
    void push (int i);    // dokada liczb na stos
    void pop();           // zdejmuje liczb ze stosu
    int  top() const;     // udostpnia warto liczby na szczycie stosu
    int& top();           // udostpnia referencj do liczby na szczycie stosu
    bool empty() const;   // czy stos jest pusty?
    size_t size() const;  // biecy rozmiar (logiczny) stosu
    Stos& operator=(Stos const&); // operator przypisania
private:
    enum { initial_capacity = 1 };    // <-- staa wyliczeniowa
    void _grow();         // <-- prywatna funkcja suca do rozszerzania stosu

    int* _tab;            // tablica z danymi
    size_t _capacity;     // rzeczywisty (= fizyczny) rozmiar tablicy
    size_t _size;         // pozorny (= logiczny) rozmiar tablicy
};

inline Stos::Stos()
: _tab( new int[initial_capacity]),
  _capacity(initial_capacity),
  _size (0)
{
    assert (initial_capacity > 0);
}

inline Stos::~Stos () { delete [] _tab; }

inline void Stos::push (int n)
{
   if (_size == _capacity)   // jeli nie ma wicej miejsca na stosie...
       _grow();
     
   _tab[_size] = n;
   _size++;
}
  
inline void Stos::pop()
{
    assert( !empty() );
    _size--;
}

inline int Stos::top() const
{
    assert( !empty() );
    return _tab[_size - 1];
}

inline int& Stos::top()
{
    assert( !empty() );
    return _tab[_size - 1];
}

inline bool Stos::empty() const { return _size == 0; }

inline size_t Stos::size() const { return _size; }



void Stos::_grow ()
{
    assert( _size == _capacity );
     
    std::cout << "zwiekszam rozmiar tablicy z " << _capacity;
    _capacity *= 2;
    std::cout << " do " << _capacity << "\n";
    int *p = new int[_capacity];
    for (size_t i = 0; i < _size; i++)
        p[i] = _tab[i];
    delete [] _tab;
    _tab = p;
}

Stos::Stos(Stos const& rhs)
: _tab( new int [rhs._size] ), _capacity(rhs._size), _size(rhs._size)
{
    for (size_t i = 0; i < _size; i++)
	_tab[i] = rhs._tab[i];
}

Stos& Stos::operator=(Stos const& rhs)
{
    if (this == &rhs)
	return *this;
	
    if (_capacity < rhs._size)
    {
       delete [] _tab;
       _capacity = rhs._size;
       _tab = new int [_capacity];
    }
    _size = rhs._size;
    for (size_t i = 0; i < _size; i++)
	_tab[i] = rhs._tab[i];
    return *this;
}


int main()
{
    Stos stos;
    for (int i = 1; i <= 50; i++)
    {
       if (i % 3 == i % 5)
          stos.push(i);
    }
    while (!stos.empty())
    {
	   std::cout << "zdejmuje ze stosu: " << stos.top() << "\n";
	   stos.pop();
    }
    system("pause");
}
